@@ -124,7 +124,7 @@ func (p *Progress) generateTrackerStrDeterminate(value int64, total int64, maxLe
 	} else if pFinishedDotsFraction == 0 {
 		pInProgress = ""
 	}
-	pFinishedStrLen := text.RuneCount(pFinished + pInProgress)
+	pFinishedStrLen := text.RuneWidthWithoutEscSequences(pFinished + pInProgress)
 	if pFinishedStrLen < maxLen {
 		pUnfinished = strings.Repeat(p.style.Chars.Unfinished, maxLen-pFinishedStrLen)
 	}
@@ -144,8 +144,8 @@ func (p *Progress) generateTrackerStrIndeterminate(maxLen int) string {
 		pUnfinished += strings.Repeat(p.style.Chars.Unfinished, indicator.Position)
 	}
 	pUnfinished += indicator.Text
-	if text.RuneCount(pUnfinished) < maxLen {
-		pUnfinished += strings.Repeat(p.style.Chars.Unfinished, maxLen-text.RuneCount(pUnfinished))
+	if text.RuneWidthWithoutEscSequences(pUnfinished) < maxLen {
+		pUnfinished += strings.Repeat(p.style.Chars.Unfinished, maxLen-text.RuneWidthWithoutEscSequences(pUnfinished))
 	}
 
 	return p.style.Colors.Tracker.Sprintf("%s%s%s",
@@ -158,9 +158,29 @@ func (p *Progress) moveCursorToTheTop(out *strings.Builder) {
 	if p.style.Visibility.TrackerOverall && p.overallTracker != nil && !p.overallTracker.IsDone() {
 		numLinesToMoveUp++
 	}
-	if numLinesToMoveUp > 0 {
-		out.WriteString(text.CursorUp.Sprintn(numLinesToMoveUp))
+	if p.style.Visibility.Pinned {
+		numLinesToMoveUp += p.pinnedMessageNumLines
 	}
+	for numLinesToMoveUp > 0 {
+		out.WriteString(text.CursorUp.Sprint())
+		out.WriteString(text.EraseLine.Sprint())
+		numLinesToMoveUp--
+	}
+}
+
+func (p *Progress) renderPinnedMessages(out *strings.Builder) {
+	p.pinnedMessageMutex.RLock()
+	defer p.pinnedMessageMutex.RUnlock()
+
+	numLines := len(p.pinnedMessages)
+	for _, msg := range p.pinnedMessages {
+		msg = strings.TrimSpace(msg)
+		out.WriteString(p.style.Colors.Pinned.Sprint(msg))
+		out.WriteRune('\n')
+
+		numLines += strings.Count(msg, "\n")
+	}
+	p.pinnedMessageNumLines = numLines
 }
 
 func (p *Progress) renderTracker(out *strings.Builder, t *Tracker, hint renderHint) {
@@ -172,7 +192,7 @@ func (p *Progress) renderTracker(out *strings.Builder, t *Tracker, hint renderHi
 		message = strings.Replace(message, "\r", "", -1)
 	}
 	if p.messageWidth > 0 {
-		messageLen := text.RuneCount(message)
+		messageLen := text.RuneWidthWithoutEscSequences(message)
 		if messageLen < p.messageWidth {
 			message = text.Pad(message, p.messageWidth, ' ')
 		} else {
@@ -180,7 +200,6 @@ func (p *Progress) renderTracker(out *strings.Builder, t *Tracker, hint renderHi
 		}
 	}
 
-	out.WriteString(text.EraseLine.Sprint())
 	if hint.isOverallTracker {
 		if !t.IsDone() {
 			hint := renderHint{hideValue: true, isOverallTracker: true}
@@ -309,6 +328,11 @@ func (p *Progress) renderTrackersDoneAndActive(out *strings.Builder) {
 	p.logsToRender = nil
 	p.logsToRenderMutex.Unlock()
 
+	// render pinned messages
+	if len(trackersActive) > 0 && p.style.Visibility.Pinned {
+		p.renderPinnedMessages(out)
+	}
+
 	// sort and render the active trackers
 	for _, tracker := range trackersActive {
 		p.renderTracker(out, tracker, renderHint{})
@@ -322,6 +346,10 @@ func (p *Progress) renderTrackerStats(out *strings.Builder, t *Tracker, hint ren
 	if !hint.hideValue || !hint.hideTime {
 		var outStats strings.Builder
 		outStats.WriteString(" [")
+
+		if p.style.Options.SpeedPosition == PositionLeft {
+			p.renderTrackerStatsSpeed(&outStats, t, hint)
+		}
 		if !hint.hideValue {
 			outStats.WriteString(p.style.Colors.Value.Sprint(t.Units.Sprint(t.Value())))
 		}
@@ -331,12 +359,55 @@ func (p *Progress) renderTrackerStats(out *strings.Builder, t *Tracker, hint ren
 		if !hint.hideTime {
 			p.renderTrackerStatsTime(&outStats, t, hint)
 		}
+		if p.style.Options.SpeedPosition == PositionRight {
+			p.renderTrackerStatsSpeed(&outStats, t, hint)
+		}
 		outStats.WriteRune(']')
 
 		out.WriteString(p.style.Colors.Stats.Sprint(outStats.String()))
 	}
 }
 
+func (p *Progress) renderTrackerStatsSpeed(out *strings.Builder, t *Tracker, hint renderHint) {
+	if hint.isOverallTracker && !p.style.Visibility.SpeedOverall {
+		return
+	}
+	if !hint.isOverallTracker && !p.style.Visibility.Speed {
+		return
+	}
+
+	speedPrecision := p.style.Options.SpeedPrecision
+	writeSpeed := func(speed string) {
+		if p.style.Options.SpeedPosition == PositionRight {
+			out.WriteString("; ")
+		}
+		out.WriteString(p.style.Colors.Speed.Sprint(speed))
+		out.WriteString(p.style.Options.SpeedSuffix)
+		if p.style.Options.SpeedPosition == PositionLeft {
+			out.WriteString("; ")
+		}
+	}
+
+	if hint.isOverallTracker {
+		speed := float64(0)
+
+		p.trackersActiveMutex.RLock()
+		for _, tracker := range p.trackersActive {
+			speed += float64(tracker.Value()) / time.Since(tracker.timeStart).Round(speedPrecision).Seconds()
+		}
+		p.trackersActiveMutex.RUnlock()
+
+		if speed > 0 {
+			writeSpeed(p.style.Options.SpeedOverallFormatter(int64(speed)))
+		}
+	} else {
+		timeTaken := time.Since(t.timeStart)
+		if timeTakenRounded := timeTaken.Round(speedPrecision); timeTakenRounded > speedPrecision {
+			writeSpeed(t.Units.Sprint(int64(float64(t.Value()) / timeTakenRounded.Seconds())))
+		}
+	}
+}
+
 func (p *Progress) renderTrackerStatsTime(outStats *strings.Builder, t *Tracker, hint renderHint) {
 	var td, tp time.Duration
 	if t.IsDone() {
