@@ -338,7 +338,7 @@ func (t *Table) analyzeAndStringifyColumn(colIdx int, col interface{}, hint rend
 	if strings.Contains(colStr, "\r") {
 		colStr = strings.Replace(colStr, "\r", "", -1)
 	}
-	return colStr
+	return fmt.Sprintf("%s%s", t.style.Format.Direction.Modifier(), colStr)
 }
 
 func (t *Table) getAlign(colIdx int, hint renderHint) text.Align {
@@ -422,7 +422,7 @@ func (t *Table) getBorderRight(hint renderHint) string {
 }
 
 func (t *Table) getColumnColors(colIdx int, hint renderHint) text.Colors {
-	if t.rowPainter != nil && hint.isRegularRow() && !t.isIndexColumn(colIdx, hint) {
+	if t.rowPainter != nil && hint.isRegularNonSeparatorRow() && !t.isIndexColumn(colIdx, hint) {
 		colors := t.rowsColors[hint.rowNumber-1]
 		if colors != nil {
 			return colors
@@ -922,6 +922,9 @@ func (t *Table) shouldMergeCellsHorizontallyBelow(row rowStr, colIdx int, hint r
 		} else if hint.isHeaderRow && hint.isLastRow {
 			rowConfig = t.getRowConfig(renderHint{rowNumber: 1})
 			row = t.getRow(0, renderHint{})
+		} else if hint.isHeaderRow {
+			rowConfig = t.getRowConfig(renderHint{isHeaderRow: true, rowNumber: hint.rowNumber + 1})
+			row = t.getRow(hint.rowNumber, hint)
 		} else if hint.isFooterRow && hint.rowNumber >= 0 {
 			rowConfig = t.getRowConfig(renderHint{isFooterRow: true, rowNumber: 1})
 			row = t.getRow(hint.rowNumber, renderHint{isFooterRow: true})
@@ -990,6 +993,10 @@ func (h *renderHint) isRegularRow() bool {
 	return !h.isHeaderRow && !h.isFooterRow
 }
 
+func (h *renderHint) isRegularNonSeparatorRow() bool {
+	return !h.isHeaderRow && !h.isFooterRow && !h.isSeparatorRow
+}
+
 func (h *renderHint) isHeaderOrFooterSeparator() bool {
 	return h.isSeparatorRow && !h.isBorderBottom && !h.isBorderTop &&
 		((h.isHeaderRow && !h.isLastRow) || (h.isFooterRow && (!h.isFirstRow || h.rowNumber > 0)))
